#ifndef SCR_DRV_H
#define SCR_DRV_H
#include "scr_cnf.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdarg.h>

//Data structures must be defined for each platform...

#if defined(NCURSES) || defined(NCURDEBUG) //duplicated for debugging
#include <ncurses.h>
typedef chtype scr_char;
typedef chtype scr_attr;
#define SCR_BOLD A_BOLD //CHECK
#define SCR_NORMAL A_NORMAL //CHECK
#define SCR_CF_BLACK 0
#define SCR_CF_RED   1
#define SCR_CF_GREEN 2
#define SCR_CF_YELLOW 3
#define SCR_CF_BLUE 4
#define SCR_CF_MAGENTA 5
#define SCR_CF_CYAN 6
#define SCR_CF_WHITE 7
#define SCR_CB_BLACK (0 << 3)
#define SCR_CB_RED   (1 << 3)
#define SCR_CB_GREEN (2  << 3)
#define SCR_CB_YELLOW (3 << 3)
#define SCR_CB_BLUE (4 << 3)
#define SCR_CB_MAGENTA (5 << 3)
#define SCR_CB_CYAN (6 << 3)
#define SCR_CB_WHITE (7 << 3)

#define SCR_KEY_NONE ERR
#define SCR_KEY_CENTER KEY_B2
#define SCR_KEY_PAUSE KEY_BREAK
#define SCR_KEY_PAGEDOWN KEY_NPAGE
#define SCR_KEY_PAGEUP KEY_PPAGE
#define SCR_KEY_DELETE KEY_DC
#define SCR_KEY_INSERT KEY_IC
#define SCR_KEY_DOWN KEY_DOWN
#define SCR_KEY_UP KEY_UP
#define SCR_KEY_LEFT KEY_LEFT
#define SCR_KEY_RIGHT KEY_RIGHT
#define SCR_KEY_HOME KEY_HOME
#define SCR_KEY_BACKSPACE KEY_BACKSPACE
#define SCR_KEY_F1 KEY_F(1)
#define SCR_KEY_F2 KEY_F(2)
#define SCR_KEY_F3 KEY_F(3)
#define SCR_KEY_F4 KEY_F(4)
#define SCR_KEY_F5 KEY_F(5)
#define SCR_KEY_F6 KEY_F(6)
#define SCR_KEY_F7 KEY_F(7)
#define SCR_KEY_F8 KEY_F(8)
#define SCR_KEY_F9 KEY_F(9)
#define SCR_KEY_F10 KEY_F(10)
#define SCR_KEY_F11 KEY_F(11)
#define SCR_KEY_F12 KEY_F(12)
#define SCR_KEY_F13 KEY_F(13)
#define SCR_KEY_F14 KEY_F(14)
#define SCR_KEY_F15 KEY_F(15)
#define SCR_KEY_F16 KEY_F(16)
#define SCR_KEY_F17 KEY_F(17)
#define SCR_KEY_F18 KEY_F(18)
#define SCR_KEY_F19 KEY_F(19)
#define SCR_KEY_F20 KEY_F(20)
#define SCR_KEY_PRINT KEY_PRINT
#define SCR_KEY_END KEY_END
#define SCR_KEY_ENTER (10+0x900)
#define SCR_KEY_ESC (27+0x900)
#define SCR_KEY_TAB (9+0x900)

#endif

#ifdef MINIXCURSES
#include <curses.h>
typedef int scr_char;
typedef int scr_attr;
#define SCR_BOLD A_BOLD //CHECK
#define SCR_NORMAL 0 //CHECK
#define SCR_CF_BLACK 0
#define SCR_CF_RED   1
#define SCR_CF_GREEN 2
#define SCR_CF_YELLOW 3
#define SCR_CF_BLUE 4
#define SCR_CF_MAGENTA 5
#define SCR_CF_CYAN 6
#define SCR_CF_WHITE 7
#define SCR_CB_BLACK (0 << 3)
#define SCR_CB_RED   (1 << 3)
#define SCR_CB_GREEN (2  << 3)
#define SCR_CB_YELLOW (3 << 3)
#define SCR_CB_BLUE (4 << 3)
#define SCR_CB_MAGENTA (5 << 3)
#define SCR_CB_CYAN (6 << 3)
#define SCR_CB_WHITE (7 << 3)
#define SCR_KEY_NONE ERR
#define SCR_KEY_CENTER 1
#define SCR_KEY_PAUSE 2 
#define SCR_KEY_PAGEDOWN 3
#define SCR_KEY_PAGEUP 4
#define SCR_KEY_DELETE 5
#define SCR_KEY_INSERT 6
#define SCR_KEY_DOWN 7
#define SCR_KEY_UP 8
#define SCR_KEY_LEFT 9
#define SCR_KEY_RIGHT 10
#define SCR_KEY_HOME 11
#define SCR_KEY_BACKSPACE 12
#define SCR_KEY_PRINT 13
#define SCR_KEY_END 14
#define SCR_KEY_ENTER (10+0x900)
#define SCR_KEY_ESC (27+0x900)
#define SCR_KEY_TAB (9+0x900)
#endif

#ifdef DOSCONIO
#include <conio.h>
#include <pc.h>
#include <keys.h>

typedef short scr_char; //why should I read source to know this?
typedef short scr_attr;

#define SCR_BOLD 0x800 //CHECK
#define SCR_NORMAL 0x00 //CHECK
#define SCR_CF_BLACK 0
#define SCR_CF_BLUE 1
#define SCR_CF_GREEN 2
#define SCR_CF_YELLOW 3
#define SCR_CF_RED 4
#define SCR_CF_MAGENTA 5
#define SCR_CF_CYAN 6
#define SCR_CF_WHITE 7

#define SCR_CB_BLACK (0 << 4)
#define SCR_CB_BLUE (1 << 4)
#define SCR_CB_GREEN (2  << 4)
#define SCR_CB_YELLOW (3 << 4)
#define SCR_CB_RED   (4 << 4)
#define SCR_CB_MAGENTA (5 << 4)
#define SCR_CB_CYAN (6 << 4)
#define SCR_CB_WHITE (7 << 4)

#define SCR_KEY_NONE (-1)
#define SCR_KEY_PRINT K_Control_Print
#define SCR_KEY_BACKSPACE K_BackSpace
#define SCR_KEY_CENTER K_Center
#define SCR_KEY_DELETE K_Delete
#define SCR_KEY_DOWN K_Down
#define SCR_KEY_END K_End
#define SCR_KEY_F1 K_F1
#define SCR_KEY_F2 K_F2
#define SCR_KEY_F3 K_F3
#define SCR_KEY_F4 K_F4
#define SCR_KEY_F5 K_F5
#define SCR_KEY_F6 K_F6
#define SCR_KEY_F7 K_F7
#define SCR_KEY_F8 K_F8
#define SCR_KEY_F9 K_F9
#define SCR_KEY_F10 K_F10
#define SCR_KEY_F11 K_F11
#define SCR_KEY_F12 K_F12
#define SCR_KEY_F13 K_Shift_F3
#define SCR_KEY_F14 K_Shift_F4
#define SCR_KEY_F15 K_Shift_F5
#define SCR_KEY_F16 K_Shift_F6
#define SCR_KEY_F17 K_Shift_F7
#define SCR_KEY_F18 K_Shift_F8
#define SCR_KEY_F19 K_Shift_F9
#define SCR_KEY_F20 K_Shift_F10
#define SCR_KEY_HOME K_Home
#define SCR_KEY_LEFT K_Left
#define SCR_KEY_PAGEDOWN K_PageDown
#define SCR_KEY_PAGEUP K_PageUp
#define SCR_KEY_RIGHT K_Right
#define SCR_KEY_UP K_Up
#define SCR_KEY_INSERT K_Insert
#define SCR_KEY_PAUSE 0x900
#define SCR_KEY_ENTER 0x901 //CHECK!!!
#define SCR_KEY_TAB 0x902
#define SCR_KEY_ESC 0x903
#endif


#ifdef WINAPIDRV
#include <windows.h>
typedef CHAR_INFO scr_char;
typedef WORD scr_attr;

#define SCR_BOLD FOREGROUND_INTENSITY //CHECK
#define SCR_CF_BLACK 0
#define SCR_CF_BLUE FOREGROUND_BLUE
#define SCR_CF_GREEN FOREGROUND_GREEN
#define SCR_CF_YELLOW (FOREGROUND_RED | FOREGROUND_GREEN)
#define SCR_CF_RED   FOREGROUND_RED
#define SCR_CF_MAGENTA (FOREGROUND_RED | FOREGROUND_BLUE)
#define SCR_CF_CYAN (FOREGROUND_BLUE | FOREGROUND_GREEN)
#define SCR_CF_WHITE (FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE)

#define SCR_CB_BLACK 0
#define SCR_CB_BLUE BACKGROUND_BLUE
#define SCR_CB_GREEN BACKGROUND_GREEN
#define SCR_CB_YELLOW (BACKGROUND_RED | BACKGROUND_GREEN)
#define SCR_CB_RED BACKGROUND_RED
#define SCR_CB_MAGENTA (BACKGROUND_RED | BACKGROUND_BLUE)
#define SCR_CB_CYAN  (BACKGROUND_BLUE | BACKGROUND_GREEN)
#define SCR_CB_WHITE (BACKGROUND_RED | BACKGROUND_GREEN | BACKGROUND_BLUE)

#define SCR_KEY_NONE (-2)
#define SCR_KEY_ENTER 0x900
#define SCR_KEY_DELETE (VK_DELETE + 0x100)
#define SCR_KEY_ESC (VK_ESCAPE + 0x100)
#define SCR_KEY_TAB (VK_TAB + 0x100)
#define SCR_KEY_DOWN (VK_DOWN + 0x100)
#define SCR_KEY_UP (VK_UP + 0x100)
#define SCR_KEY_LEFT (VK_LEFT + 0x100)
#define SCR_KEY_RIGHT (VK_RIGHT + 0x100)
#define SCR_KEY_HOME (VK_HOME + 0x100)
#define SCR_KEY_BACKSPACE (VK_BACK + 0x100)
#define SCR_KEY_F1 (VK_F1 + 0x100)
#define SCR_KEY_F2 (VK_F2 + 0x100)
#define SCR_KEY_F3 (VK_F3 + 0x100)
#define SCR_KEY_F4 (VK_F4 + 0x100)
#define SCR_KEY_F5 (VK_F5 + 0x100)
#define SCR_KEY_F6 (VK_F6 + 0x100)
#define SCR_KEY_F7 (VK_F7 + 0x100)
#define SCR_KEY_F8 (VK_F8 + 0x100)
#define SCR_KEY_F9 (VK_F9 + 0x100)
#define SCR_KEY_F10 (VK_F10 + 0x100)
#define SCR_KEY_F11 (VK_F11 + 0x100)
#define SCR_KEY_F12 (VK_F12 + 0x100)
#define SCR_KEY_F13 (VK_F13 + 0x100)
#define SCR_KEY_F14 (VK_F14 + 0x100)
#define SCR_KEY_F15 (VK_F15 + 0x100)
#define SCR_KEY_F16 (VK_F16 + 0x100)
#define SCR_KEY_F17 (VK_F17 + 0x100)
#define SCR_KEY_F18 (VK_F18 + 0x100)
#define SCR_KEY_F19 (VK_F19 + 0x100)
#define SCR_KEY_F20 (VK_F20 + 0x100)
#define SCR_KEY_INSERT (VK_INSERT + 0x100)
#define SCR_KEY_END (VK_END + 0x100)
#define SCR_KEY_CENTER (VK_CLEAR + 0x100)
#define SCR_KEY_PAGEUP (VK_PRIOR + 0x100)
#define SCR_KEY_PAGEDOWN (VK_NEXT + 0x100)
#define SCR_KEY_PRINT (VK_PRINT + 0x100)
#define SCR_KEY_PAUSE (VK_PAUSE + 0x100)
#endif

//These functions must be written for each new driver/platform...:

 void scr_refresh(void);
 //this function should refresh to physical screen AND position terminal
 //cursor to scr_curs_x,scr_curs_y.
 void scr_initscr_low(void); //inits just the character device, not memory
 void scr_stopscr_low(void); //deinits just the character device
 void scr_cursor(unsigned char x,unsigned char y); //JUST! after refresh
 // scr_curs_x,scr_curs_y should be corrected immediately, anyway
 void scr_hidecursor(void); //hide physical cursor
 void scr_showcursor(void); //show physical cursor
 int scr_getch(void); //return char or one of SCR_KEY constants!
 int scr_getch_nowait(void); //see above... and SCR_KEY_NONE
 unsigned char scr_maxx(void); //get screen width (for ex., 80 for 80x25)
 unsigned char scr_maxy(void); //get screen heigth (for ex., 25 for 80x25)

//type conversion functions:
 unsigned char scr_char2char(scr_char ch); //extracts char from scr_char
 scr_attr scr_char2attr(scr_char ch);//extracts scr_attr from scr_char
 scr_char scr_to_scr_char(unsigned char ch, scr_attr attr); //makes scr_char
 scr_attr scr_color2attr(unsigned int color); //makes attr from color
//scr_char comparision function:
 unsigned char scr_charcomp(scr_char ch1,scr_char ch2);

//just for some platforms. Used by predefined methods.
//may be, put some ifdefs here?

//for linewise method:
 void scr_drv_showline(unsigned char x, unsigned char y,
              unsigned char xlen, scr_char* line);
//should display a line to physical screen
//for 'teletype' method:
 void scr_drv_home(void);  //position to 0:0
 void scr_drv_skiplines(unsigned char lines); //skip so much lines
 void scr_drv_skipchars(unsigned char chars); //skip so much chars
 void scr_drv_showstring(unsigned char len, scr_char* line);

#endif
