#include "scr_ind.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdarg.h>

//This is an abstract, portable but very simple interface to screen
//It's lowest level platform-INDEPENDENT part 
//It replaces ncurses, has similar API
//It's compiled in if ncurses aren't.
//Ncurses covers both this part and platform-dependend one, so it lacks
//functions which MUST be exported by all other drivers
//platform-dependent part is handled by appropriate drivers

char scr_area[200]; //buffer for scr_printf.
unsigned char scr_curr_x=0;
unsigned char scr_curr_y=0;
scr_attr scr_curr_attr;
unsigned char scr_curs_x=0;
unsigned char scr_curs_y=0; //cursor coords
unsigned char scr_width=80;
unsigned char scr_height=25;

static void scr_advancecursor(void);
static void scr_advancecursor(void)
{
//advance output point
scr_curr_x++;
scr_curr_y += scr_curr_x/scr_width;
scr_curr_x %= scr_width;
}

#ifdef NCURSES
//arrays for recoding of chars. Remake them.
static unsigned char SCR_LDS[]={
      0x82, 0x84, 0x83, 0x85, 0x86, 0x87, 0x89, 0x88, 0x80, 0x81, 0x8a, 0x7e,
      0x5f, 0x94, 0x91, 0x9c, 0x94, 0x94, 0x94, 0x94, 0x94, 0x94, 0x90, 0xbe,
      0x8d, 0x80, 0x80, 0x98, 0x99, 0x94, 0xbc, 0x94, 0x82, 0x84, 0x83, 0x85,
      0x87, 0x86, 0x89, 0x88, 0x80, 0x81, 0x8a, 0x00};
static scr_char SCR_SPECCHRS[43];

unsigned char scr_maxx(void) 
{
int row,col;
getmaxyx(stdscr,row,col);    
return (unsigned char) col;    
} //screen width

unsigned char scr_maxy(void) //screen heigth
{
int row,col;
getmaxyx(stdscr,row,col);    
return (unsigned char) row;    
}

int scr_getch(void) 
{
int toget;
toget=ERR;
while (toget==ERR) toget=getch();
if (toget==10) return SCR_KEY_ENTER;
if (toget==27) return SCR_KEY_ESC;
if (toget==9) return SCR_KEY_TAB;
return toget;
} //think about it, damn it

int scr_getch_nowait(void) 
{
int toget;
(void) nodelay(stdscr, TRUE);
toget=getch();
(void) nodelay(stdscr, FALSE);
if (toget==10) return SCR_KEY_ENTER;
if (toget==27) return SCR_KEY_ESC;
if (toget==9) return SCR_KEY_TAB;
return toget;
}

void scr_refresh(void)
{
(void) move(scr_curs_y, scr_curs_x);
(void) refresh();
}

void scr_addch(scr_char ch)
{
(void) addch(ch);
scr_advancecursor();
}

void scr_mvaddch(unsigned char x, unsigned char y, scr_char ch)
{
scr_curr_x=x;
scr_curr_y=y;
//(void) mvaddch(y,x,ch);
(void) mvaddchnstr(y,x,&ch,1);
scr_advancecursor();
}

scr_char scr_inch(void)
{
scr_advancecursor();
return inch();
}

void scr_cursor(unsigned char x,unsigned char y)
{
scr_curs_x=x;
scr_curs_y=y;
}

void scr_initscr(void)
{
(void) initscr();
(void) clear();
(void) noecho();
(void) cbreak(); /* Line buffering disabled. Pass on everything */
(void) meta(stdscr, TRUE);
(void) keypad(stdscr,TRUE);
timeout(50);
wtimeout(stdscr, 50);
(void) leaveok(stdscr,0);
(void) start_color();
(void) init_pair(SCR_CF_BLACK | SCR_CB_RED, COLOR_BLACK, COLOR_RED);
(void) init_pair(SCR_CF_BLACK | SCR_CB_GREEN, COLOR_BLACK, COLOR_GREEN);
(void) init_pair(SCR_CF_BLACK | SCR_CB_YELLOW, COLOR_BLACK, COLOR_YELLOW);
(void) init_pair(SCR_CF_BLACK | SCR_CB_BLUE, COLOR_BLACK, COLOR_BLUE);
(void) init_pair(SCR_CF_BLACK | SCR_CB_MAGENTA, COLOR_BLACK, COLOR_MAGENTA);
(void) init_pair(SCR_CF_BLACK | SCR_CB_CYAN, COLOR_BLACK, COLOR_CYAN);
(void) init_pair(SCR_CF_BLACK | SCR_CB_WHITE, COLOR_BLACK, COLOR_WHITE);
(void) init_pair(SCR_CF_RED | SCR_CB_BLACK, COLOR_RED, COLOR_BLACK);
(void) init_pair(SCR_CF_RED | SCR_CB_RED, COLOR_RED, COLOR_RED);
(void) init_pair(SCR_CF_RED | SCR_CB_GREEN, COLOR_RED, COLOR_GREEN);
(void) init_pair(SCR_CF_RED | SCR_CB_YELLOW, COLOR_RED, COLOR_YELLOW);
(void) init_pair(SCR_CF_RED | SCR_CB_BLUE, COLOR_RED, COLOR_BLUE);
(void) init_pair(SCR_CF_RED | SCR_CB_MAGENTA, COLOR_RED, COLOR_MAGENTA);
(void) init_pair(SCR_CF_RED | SCR_CB_CYAN, COLOR_RED, COLOR_CYAN);
(void) init_pair(SCR_CF_RED | SCR_CB_WHITE, COLOR_RED, COLOR_WHITE);
(void) init_pair(SCR_CF_GREEN | SCR_CB_BLACK, COLOR_GREEN, COLOR_BLACK);
(void) init_pair(SCR_CF_GREEN | SCR_CB_RED, COLOR_GREEN, COLOR_RED);
(void) init_pair(SCR_CF_GREEN | SCR_CB_GREEN, COLOR_GREEN, COLOR_GREEN);
(void) init_pair(SCR_CF_GREEN | SCR_CB_YELLOW, COLOR_GREEN, COLOR_YELLOW);
(void) init_pair(SCR_CF_GREEN | SCR_CB_BLUE, COLOR_GREEN, COLOR_BLUE);
(void) init_pair(SCR_CF_GREEN | SCR_CB_MAGENTA, COLOR_GREEN, COLOR_MAGENTA);
(void) init_pair(SCR_CF_GREEN | SCR_CB_CYAN, COLOR_GREEN, COLOR_CYAN);
(void) init_pair(SCR_CF_GREEN | SCR_CB_WHITE, COLOR_GREEN, COLOR_WHITE);
(void) init_pair(SCR_CF_YELLOW | SCR_CB_BLACK, COLOR_YELLOW, COLOR_BLACK);
(void) init_pair(SCR_CF_YELLOW | SCR_CB_RED, COLOR_YELLOW, COLOR_RED);
(void) init_pair(SCR_CF_YELLOW | SCR_CB_GREEN, COLOR_YELLOW, COLOR_GREEN);
(void) init_pair(SCR_CF_YELLOW | SCR_CB_YELLOW, COLOR_YELLOW, COLOR_YELLOW);
(void) init_pair(SCR_CF_YELLOW | SCR_CB_BLUE, COLOR_YELLOW, COLOR_BLUE);
(void) init_pair(SCR_CF_YELLOW | SCR_CB_MAGENTA, COLOR_YELLOW, COLOR_MAGENTA);
(void) init_pair(SCR_CF_YELLOW | SCR_CB_CYAN, COLOR_YELLOW, COLOR_CYAN);
(void) init_pair(SCR_CF_YELLOW | SCR_CB_WHITE, COLOR_YELLOW, COLOR_WHITE);
(void) init_pair(SCR_CF_BLUE | SCR_CB_BLACK, COLOR_BLUE, COLOR_BLACK);
(void) init_pair(SCR_CF_BLUE | SCR_CB_RED, COLOR_BLUE, COLOR_RED);
(void) init_pair(SCR_CF_BLUE | SCR_CB_GREEN, COLOR_BLUE, COLOR_GREEN);
(void) init_pair(SCR_CF_BLUE | SCR_CB_YELLOW, COLOR_BLUE, COLOR_YELLOW);
(void) init_pair(SCR_CF_BLUE | SCR_CB_BLUE, COLOR_BLUE, COLOR_BLUE);
(void) init_pair(SCR_CF_BLUE | SCR_CB_MAGENTA, COLOR_BLUE, COLOR_MAGENTA);
(void) init_pair(SCR_CF_BLUE | SCR_CB_CYAN, COLOR_BLUE, COLOR_CYAN);
(void) init_pair(SCR_CF_BLUE | SCR_CB_WHITE, COLOR_BLUE, COLOR_WHITE);
(void) init_pair(SCR_CF_MAGENTA | SCR_CB_BLACK, COLOR_MAGENTA, COLOR_BLACK);
(void) init_pair(SCR_CF_MAGENTA | SCR_CB_RED, COLOR_MAGENTA, COLOR_RED);
(void) init_pair(SCR_CF_MAGENTA | SCR_CB_GREEN, COLOR_MAGENTA, COLOR_GREEN);
(void) init_pair(SCR_CF_MAGENTA | SCR_CB_YELLOW, COLOR_MAGENTA, COLOR_YELLOW);
(void) init_pair(SCR_CF_MAGENTA | SCR_CB_BLUE, COLOR_MAGENTA, COLOR_BLUE);
(void) init_pair(SCR_CF_MAGENTA | SCR_CB_MAGENTA, COLOR_MAGENTA, COLOR_MAGENTA);
(void) init_pair(SCR_CF_MAGENTA | SCR_CB_CYAN, COLOR_MAGENTA, COLOR_CYAN);
(void) init_pair(SCR_CF_MAGENTA | SCR_CB_WHITE, COLOR_MAGENTA, COLOR_WHITE);
(void) init_pair(SCR_CF_CYAN | SCR_CB_BLACK, COLOR_CYAN, COLOR_BLACK);
(void) init_pair(SCR_CF_CYAN | SCR_CB_RED, COLOR_CYAN, COLOR_RED);
(void) init_pair(SCR_CF_CYAN | SCR_CB_GREEN, COLOR_CYAN, COLOR_GREEN);
(void) init_pair(SCR_CF_CYAN | SCR_CB_YELLOW, COLOR_CYAN, COLOR_YELLOW);
(void) init_pair(SCR_CF_CYAN | SCR_CB_BLUE, COLOR_CYAN, COLOR_BLUE);
(void) init_pair(SCR_CF_CYAN | SCR_CB_MAGENTA, COLOR_CYAN, COLOR_MAGENTA);
(void) init_pair(SCR_CF_CYAN | SCR_CB_CYAN, COLOR_CYAN, COLOR_CYAN);
(void) init_pair(SCR_CF_CYAN | SCR_CB_WHITE, COLOR_CYAN, COLOR_WHITE);
(void) init_pair(SCR_CF_WHITE | SCR_CB_BLACK, COLOR_WHITE, COLOR_BLACK);
(void) init_pair(SCR_CF_WHITE | SCR_CB_RED, COLOR_WHITE, COLOR_RED);
(void) init_pair(SCR_CF_WHITE | SCR_CB_GREEN, COLOR_WHITE, COLOR_GREEN);
(void) init_pair(SCR_CF_WHITE | SCR_CB_YELLOW, COLOR_WHITE, COLOR_YELLOW);
(void) init_pair(SCR_CF_WHITE | SCR_CB_BLUE, COLOR_WHITE, COLOR_BLUE);
(void) init_pair(SCR_CF_WHITE | SCR_CB_MAGENTA, COLOR_WHITE, COLOR_MAGENTA);
(void) init_pair(SCR_CF_WHITE | SCR_CB_CYAN, COLOR_WHITE, COLOR_CYAN);
(void) init_pair(SCR_CF_WHITE | SCR_CB_WHITE, COLOR_WHITE, COLOR_WHITE);
//now init special characters array:
SCR_SPECCHRS[0]=ACS_ULCORNER;
SCR_SPECCHRS[1]=ACS_LLCORNER;
SCR_SPECCHRS[2]=ACS_URCORNER;
SCR_SPECCHRS[3]=ACS_LRCORNER;
SCR_SPECCHRS[4]=ACS_LTEE;
SCR_SPECCHRS[5]=ACS_RTEE;
SCR_SPECCHRS[6]=ACS_BTEE;
SCR_SPECCHRS[7]=ACS_TTEE;
SCR_SPECCHRS[8]=ACS_HLINE;
SCR_SPECCHRS[9]=ACS_VLINE;
SCR_SPECCHRS[10]=ACS_PLUS;
SCR_SPECCHRS[11]=ACS_S1;
SCR_SPECCHRS[12]=ACS_S9;
SCR_SPECCHRS[13]=ACS_DIAMOND;
SCR_SPECCHRS[14]=ACS_CKBOARD;
SCR_SPECCHRS[15]=ACS_DEGREE;
SCR_SPECCHRS[16]=ACS_PLMINUS;
SCR_SPECCHRS[17]=ACS_BULLET;
SCR_SPECCHRS[18]=ACS_LARROW;
SCR_SPECCHRS[19]=ACS_RARROW;
SCR_SPECCHRS[20]=ACS_DARROW;
SCR_SPECCHRS[21]=ACS_UARROW;
SCR_SPECCHRS[22]=ACS_BOARD;
SCR_SPECCHRS[23]=ACS_LANTERN;
SCR_SPECCHRS[24]=ACS_BLOCK;
SCR_SPECCHRS[25]=ACS_S3;
SCR_SPECCHRS[26]=ACS_S7;
SCR_SPECCHRS[27]=ACS_LEQUAL;
SCR_SPECCHRS[28]=ACS_GEQUAL;
SCR_SPECCHRS[29]=ACS_PI;
SCR_SPECCHRS[30]=ACS_NEQUAL;
SCR_SPECCHRS[31]=ACS_STERLING;
SCR_SPECCHRS[32]=ACS_BSSB;
SCR_SPECCHRS[33]=ACS_SSBB;
SCR_SPECCHRS[34]=ACS_BBSS;
SCR_SPECCHRS[35]=ACS_SBBS;
SCR_SPECCHRS[36]=ACS_SBSS;
SCR_SPECCHRS[37]=ACS_SSSB;
SCR_SPECCHRS[38]=ACS_SSBS;
SCR_SPECCHRS[39]=ACS_BSSS;
SCR_SPECCHRS[40]=ACS_BSBS;
SCR_SPECCHRS[41]=ACS_SBSB;
SCR_SPECCHRS[42]=ACS_SSSS;
//--------------------
(void) clear();
(void) move(0,0);
scr_width=scr_maxx();
scr_height=scr_maxy();
scr_curr_x=0;
scr_curr_y=0;
scr_curr_attr=scr_color2attr(SCR_CF_WHITE | SCR_CB_BLACK);
(void) refresh();
}

scr_char scr_mvinch(unsigned char x, unsigned char y)
{
scr_char thechar;
scr_curr_x=x;
scr_curr_y=y;
thechar=mvinch(y,x);
scr_advancecursor();
return thechar; 
}

void scr_hidecursor(void) 
{
(void) curs_set(0);
}

void scr_showcursor(void)
{
(void) curs_set(1);
}

void scr_cls(void)
{
(void) clear();
(void) move(0,0);
scr_curs_x=0;
scr_curs_y=0;
scr_curr_x=0;
scr_curr_y=0;
}

void scr_locate(unsigned char x,unsigned char y)
{
(void) move(y,x);
scr_curr_x=x;
scr_curr_y=y;
}

void scr_stopscr(void)
{
(void) clrtoeol();
(void) refresh();
(void) endwin();
}

void scr_setattr(scr_attr attr)
{
(void) attrset(attr);
scr_curr_attr=attr;
}

//type conversion functions:
unsigned char scr_char2char(scr_char ch)
{
//CORRECT FOR LINES HERE!!!
unsigned int f;
scr_char temp;
temp=(ch & A_CHARTEXT);
for (f=0;f<(sizeof(SCR_LDS)-1);f++) {
    if (SCR_SPECCHRS[f]==temp) {
	return SCR_LDS[f]; 
	}
    };
return (unsigned char) temp;
}

scr_attr scr_char2attr(scr_char ch)
{
return (ch & A_ATTRIBUTES);
}

scr_char scr_to_scr_char(unsigned char ch, scr_attr attr)
{
//CORRECT FOR LINES HERE!!!
//must translate line-drawing chars to special ncurses macroses
unsigned int f;
for (f=0;f<(sizeof(SCR_LDS)-1);f++) {
    if (SCR_LDS[f]==ch) {
	return (attr | SCR_SPECCHRS[f]);
	}
    };
return (attr | ch);
}

scr_attr scr_color2attr(unsigned int color)
{
return (COLOR_PAIR(color));
}

unsigned char scr_charcomp(scr_char ch1,scr_char ch2)
{
return (ch1==ch2);
}

#else
/* NOT ncurses */
scr_char* scr_screen_old;
scr_char* scr_screen_new;

//scr_refresh is platform-dependend, but some of them can call
//this functions, if they use the same (or alike) output method.

//platform-independent, based on platform-dependend scr_showline
void scr_refresh_ind_linewise(void)
{
unsigned char isinline,equal;
unsigned char f,g,llen,xoffset=0;
scr_char* linep; 

for (f=0;f<scr_height;f++) {
    isinline = llen = 0;
    for (g=0;g<scr_width;g++) {
	equal=scr_charcomp(scr_screen_old[f*scr_width+g],
                           scr_screen_new[f*scr_width+g]);
        if (!equal)
        {
        scr_screen_old[f*scr_width+g]=scr_screen_new[f*scr_width+g]; //assign
	llen++; //grow current line or start a new one
	if (!isinline) {
	    isinline=1;
	    xoffset=g;
	    };
	} else { //equal
	if (isinline) 
	    {
	    //must show it now... 
	    linep=scr_screen_new+(f*scr_width+xoffset);
            scr_drv_showline(xoffset, f, llen, linep);
	    isinline = llen = 0;
	    }; //else just skip it
	};
    };
    if (isinline) 
	{
	//must show it now... 
	linep=scr_screen_new+(f*scr_width+xoffset);
	scr_drv_showline(xoffset, f, llen, linep);
	}; //else just skip it
};
}

//A cursor positioning commands exist here: scr_drv_home,
//skipchars(n), skiplines(n), etc. Goes Left->Right, Up->Down
void scr_refresh_ind_teletype(void)
{
unsigned char isinline,equal;
unsigned char f,g,llen,xskip,yskip;
scr_char* linep;

scr_drv_home();  //position to 0:0
yskip = 0;
for (f=0;f<scr_height;f++) {
    isinline = llen = xskip = 0;
    for (g=0;g<scr_width;g++) {
	equal=scr_charcomp(scr_screen_old[f*scr_width+g],
                           scr_screen_new[f*scr_width+g]);
        if (!equal)
        {
        scr_screen_old[f*scr_width+g]=scr_screen_new[f*scr_width+g]; //assign
	llen++; //grow current line or start a new one
	if (!isinline) {
	    isinline=1;
	    };
	} else { //equal
	if (isinline) 
	    {
	    //must show it now... 
	    linep=scr_screen_new+(f*scr_width+xskip);
	    if (yskip) scr_drv_skiplines(yskip);
            if (xskip) scr_drv_skipchars(xskip); //skip so much chars
            scr_drv_showstring(llen, linep);
	    isinline = llen = xskip = yskip = 0;
	    } else xskip++; //else just skip it
	};
    };
    if (isinline) 
	{
	//in the end of the string, must show it now... 
	linep=scr_screen_new+(f*scr_width+xskip);
	if (yskip) scr_drv_skiplines(yskip);
	if (xskip) scr_drv_skipchars(xskip); //skip so much chars
	scr_drv_showstring(llen, linep);
	isinline = llen = xskip = yskip = 0;
	};
    yskip++;
};
}

//platform-independent, based on platform-dependend scr_initscr_low
void scr_initscr(void)
{
scr_initscr_low(); //CHECK
//Here, we should just clear PHYSICAL screen, so just do it!
scr_width=scr_maxx();
scr_height=scr_maxy();
scr_curr_x=0;
scr_curr_y=0;
scr_curr_attr=scr_color2attr(SCR_CF_WHITE | SCR_CB_BLACK);
scr_screen_old=malloc( sizeof(scr_char) * scr_width * scr_height);
assert (scr_screen_old!=NULL);
scr_screen_new=malloc( sizeof(scr_char) * scr_width * scr_height);
assert (scr_screen_new!=NULL);
//allocated, now clear: 
scr_cls();
//Then, refresh 
scr_refresh();
}

void scr_addch(scr_char ch)
{
assert(scr_curr_y<scr_height);
assert(scr_curr_x<scr_width);
scr_screen_new[scr_curr_y*scr_width+scr_curr_x]=ch;
scr_advancecursor();
}

void scr_mvaddch(unsigned char x, unsigned char y, scr_char ch)
{
assert(y<scr_height);
assert(x<scr_width);
scr_curr_x=x;
scr_curr_y=y;
scr_screen_new[y*scr_width+x]=ch;
scr_advancecursor();
}

scr_char scr_inch(void)
{
scr_char toret;
toret=scr_screen_new[scr_curr_y*scr_width+scr_curr_x];
scr_advancecursor();
return toret;
}

scr_char scr_mvinch(unsigned char x, unsigned char y)
{
scr_char toret;
assert(y<scr_height);
assert(x<scr_width);
scr_curr_x=x;
scr_curr_y=y;
toret=scr_screen_new[y*scr_width+x];
scr_advancecursor();
return toret;
}

//should fill screen with spaces of current attr, locate(0,0)
void scr_cls(void)
{
scr_char fillchar;
unsigned char f,g;
fillchar=scr_to_scr_char(' ',scr_curr_attr);
for (f=0;f<scr_height;f++)
    for (g=0;g<scr_width;g++) {
    scr_screen_new[f*scr_width+g]=fillchar;
    };
scr_locate(0,0);
scr_cursor(0,0);
}

void scr_locate(unsigned char x,unsigned char y)
{
assert(y<scr_height);
assert(x<scr_width);
scr_curr_x=x;
scr_curr_y=y;
}

//platform independent, uses scr_stopscr_low
void scr_stopscr(void)
{
scr_stopscr_low();
free(scr_screen_old);
free(scr_screen_new);
}

void scr_setattr(scr_attr attr)
{
scr_curr_attr=attr;
}
#endif

scr_attr scr_getattr(void)
{
return scr_curr_attr;
}

//==========================================================================

//absolutely independent, but can be rewritten to be more effective 
//(but who cares?)
scr_char* scr_getarea(unsigned char x, unsigned char y, 
	          unsigned char lenx, unsigned char leny)
{
scr_char* tmparea;
unsigned char f;
unsigned char g;
assert(x<scr_width);
assert(lenx<=scr_width);
assert((x+lenx)<=scr_width); 
assert(y<scr_height);
assert(leny<=scr_height);
assert((y+leny)<=scr_height); 

if ((lenx==0) || (leny==0)) return 0;
tmparea=malloc( sizeof(scr_char) * lenx * leny );
if (!tmparea) return NULL;
for (f=0;f<leny;f++) 
   for (g=0;g<lenx;g++) 
    {
    tmparea[f*lenx+g]=(scr_char) scr_mvinch(x+g,y+f);
    };
return tmparea;
}

void scr_regetarea(unsigned char x, unsigned char y, 
	          unsigned char lenx, unsigned char leny, scr_char* area)
{
unsigned char f;
unsigned char g;
assert(x<scr_width);
assert(lenx<=scr_width);
assert((x+lenx)<=scr_width); 
assert(y<scr_height);
assert(leny<=scr_height);
assert((y+leny)<=scr_height); 

if ((lenx==0) || (leny==0)) return;
for (f=0;f<leny;f++) 
   for (g=0;g<lenx;g++) 
    {
    area[f*lenx+g]=(scr_char) scr_mvinch(x+g,y+f);
    };
}

//absolutely independent, but can be rewritten to be more effective 
//(but who cares?)
void scr_putarea(unsigned char x, unsigned char y, 
	          unsigned char lenx, unsigned char leny, scr_char* area)
{
unsigned char f;
unsigned char g;
assert(x<scr_width);
assert(lenx<=scr_width);
assert((x+lenx)<=scr_width); 
assert(y<scr_height);
assert(leny<=scr_height);
assert((y+leny)<=scr_height); 
if ((lenx==0) || (leny==0)) return;
if (!area) return;
for (f=0;f<leny;f++) 
   for (g=0;g<lenx;g++) 
    {
    (void) scr_mvaddch(x+g, y+f, area[f*lenx+g]);
    };
}

void scr_freearea(scr_char* area)
{
free(area);
}

//this is independent, just fixes DJGPP asbsence of ANSI vsnprintf
void scr_printf(char *format, ...)
{
unsigned char f;
scr_char toprnt;
va_list ap;
va_start (ap, format);
assert(format != NULL); //shouldn't be NULL
 #ifdef DOSCONIO
 vsprintf(scr_area,format,ap);
 #else
 vsnprintf(scr_area,scr_width,format,ap);
 #endif
//add check for going out of the screen
f=0;
while ((f<scr_width) && (scr_area[f]!='\0'))
{
toprnt=scr_to_scr_char(scr_area[f],scr_curr_attr);
scr_mvaddch(scr_curr_x+f,scr_curr_y,toprnt);
f++;
}
va_end (ap);
}

void scr_mvprintf(unsigned char x, unsigned char y, char* format, ...)
{
unsigned char f;
scr_char toprnt;
va_list ap;
va_start (ap, format);
assert(format != NULL); //shouldn't be NULL
 #ifdef DOSCONIO
 vsprintf(scr_area,format,ap);
 #else
 vsnprintf(scr_area,scr_width,format,ap);
 #endif
//add check for going out of screen
f=0;
while ((f<scr_width) && (scr_area[f]!='\0')) {
    toprnt=scr_to_scr_char(scr_area[f],scr_curr_attr);
    scr_mvaddch(x+f,y,toprnt);
    f++;
    }
va_end (ap);
}

