#ifndef SCR_IND_H
#define SCR_IND_H

#include "scr_drv.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdarg.h>

//This is an abstract, portable but very simple interface to screen
//It's lowest level platform-INDEPENDENT part 
//It replaces ncurses, has similar API
//It's compiled in if ncurses aren't.
//Ncurses covers both this part and platform-dependend one, so it lacks
//functions which MUST be exported by all other drivers
//platform-dependent part is handled by appropriate drivers

extern char scr_area[200]; //buffer for scr_printf. Principally, should be 
			 //dynamically allocated. We'll see.
extern unsigned char scr_curr_x;
extern unsigned char scr_curr_y;
extern scr_attr scr_curr_attr;
extern unsigned char scr_curs_x;
extern unsigned char scr_curs_y; //cursor coords
extern unsigned char scr_width;
extern unsigned char scr_height;

/* NOT ncurses */
#ifndef NCURSES
extern scr_char* scr_screen_old;
extern scr_char* scr_screen_new;
#endif

//scr_refresh is platform-dependend, but some of them can call
//this functions, if they use the same (or alike) output method.

//platform-independent, based on platform-dependend scr_showline
 void scr_refresh_ind_linewise(void);

//Here exist a cursor positioning commands: scr_drv_home,
//skipchars(n), skiplines(n), etc. Goes Left->Right, Up->Down
 void scr_refresh_ind_teletype(void);

//platform-independent, based on platform-dependend scr_initscr_low
 void scr_initscr(void);
 void scr_addch(scr_char ch);
 void scr_mvaddch(unsigned char x, unsigned char y, scr_char ch);
 scr_char scr_inch(void);
 scr_char scr_mvinch(unsigned char x, unsigned char y);
 //should fill screen with spaces of current attr, locate(0,0)
 void scr_cls(void);
 void scr_locate(unsigned char x,unsigned char y);
 //platform independent, uses scr_stopscr_low
 void scr_stopscr(void);
 void scr_setattr(scr_attr attr);
 scr_attr scr_getattr(void);
 scr_char* scr_getarea(unsigned char x, unsigned char y, 
     	      unsigned char lenx, unsigned char leny);
 void scr_regetarea(unsigned char x, unsigned char y, 
	          unsigned char lenx, unsigned char leny, scr_char* area);
 void scr_putarea(unsigned char x, unsigned char y, 
     	      unsigned char lenx, unsigned char leny, scr_char* area);
 void scr_freearea(scr_char* area);
 void scr_printf(char *format, ...);
 void scr_mvprintf(unsigned char x, unsigned char y, char* format, ...);

#endif
