#ifndef SCR_ML_H
#define SCR_ML_H
#include "scr_ind.h"
#include "scr_drv.h"

#define SCR_WT_MENU 1
#define SCR_WT_LABEL 2
#define SCR_WT_MLABEL 3
#define SCR_WT_EDIT 4
#define SCR_WT_MEMO 5
#define SCR_WT_CHECKGRP 6
#define SCR_WT_HPROGRBAR 7
#define SCR_WT_VPROGRBAR 8
#define SCR_WT_HSCROLLBAR 9
#define SCR_WT_VSCROLLBAR 10
#define SCR_WT_VIEWER 11

#define SCR_EC_OK 1
#define SCR_EC_ENTER 2
#define SCR_EC_ESC 3
#define SCR_EC_UNK 4

typedef struct {
unsigned char widgtype;
unsigned char x;
unsigned char y;
unsigned char xlen;
unsigned char ylen;
scr_attr color; //color for this widget
//scr_char *storearea;
} Scr_header;

typedef struct {
Scr_header header;
} Scr_Abstract; //for accessing any type of widget for header info...

typedef struct {
unsigned char widgtype;
void (*deallocator)(Scr_Abstract widget);
} Scr_deallocator; //NEW proposition

typedef struct {
Scr_header header;
char* text;
} Scr_label;

typedef struct { //ylen is the number of lines
Scr_header header;
char** lines;
} Scr_mlabel;

typedef struct {
Scr_header header; //maximum length is taken from header...
char* text;
unsigned char currpos; //current cursor position
unsigned char exittype; //copy of the exit code after injectchar
//pointer to function for preprocess filtering:
int (*prefilter)(int thechar);
} Scr_edit;

typedef struct {
Scr_header header; //length is taken from header...
char* text;
char* templ; //template, better to provide generator of these...
unsigned char currpos; //current cursor position
unsigned char exittype; //copy of the exit code after injectchar
//pointer to function for preprocess filtering
int (*prefilter)(int thechar);
} Scr_tedit;

typedef struct {
Scr_header header; //LENGTHS is taken from header. Width of all strings must
                   //be the same 
char** lines;
unsigned char currline; //current cursor position
unsigned char currpos; //current cursor position
unsigned char exittype; //copy of the exit code after injectchar
//unsigned char currlen; //possibly...
int (*prefilter)(int thechar);
} Scr_memo;

typedef struct {
Scr_header header;
char** choices;
unsigned char numchoices;
unsigned char currchoice; //current cursor position
unsigned char exittype; //copy of the exit code after injectchar
scr_attr selattr; //attribute of the selected choice
//pointer to function for preprocess filtering
} Scr_menu;

/*
typedef struct {
Scr_header header;
char** choices;
unsigned char numchoices;
unsigned char currchoice; //current cursor position
unsigned char chosen; //currently chosen one
unsigned char exittype; //copy of the exit code after injectchar
scr_attr selattr; //attribute of the selected choice
//pointer to function for preprocess filtering
} Scr_radio;
*/ //later

typedef struct {
Scr_header header; //lengthS are taken from here.
char** choices;
unsigned char currchoice; //current cursor position
unsigned char exittype; //copy of the exit code after injectchar
char* chosens; //mask of selected fields
scr_attr selattr; //attribute of the selected choice
//pointer to function for preprocess filtering
int (*prefilter)(int thechar);
//m/b pointer to function for displaying a choice?
} Scr_checkgrp;

typedef struct {
Scr_header header; //length is taken from here. 
unsigned char currpos; //current bar position
} Scr_hprogrbar;

typedef struct {
Scr_header header; //length is taken from here. 
unsigned char currpos; //current bar position
} Scr_vprogrbar;

typedef struct {
Scr_header header; //length is taken from here. 
unsigned char currpos; //current bar position
} Scr_hscrollbar;

typedef struct {
Scr_header header; //length is taken from here. 
unsigned char currpos; //current bar position
} Scr_vscrollbar;

typedef struct {
Scr_header header; //length is taken from here.
unsigned char options; //various flags, later...
unsigned char numitems; //how much items are SEEN
unsigned long int selected; //IT's an absolute position in list!
                           //(including calls to next/prev item!
unsigned long int selscr; //IT's an onscreen position in list!
char** choices; //allocated? and freed by user... 
scr_attr selattr; //attribute of the ... 
// pointer to function to get previous element. It returns a char*,
// And a COPY of it must be made into this widget
// pointer to function to get next element. It returns a char*,
// And a COPY of it must be made into this widget
} Scr_itemlist;

typedef struct {
Scr_header header; //length is taken from here.
unsigned int startline; //IT's an absolute position in list!
unsigned int numlines; //IT's an absolute position in list!
char** lines; //allocated and freed by user... 
unsigned char exittype; //copy of the exit code after injectchar
} Scr_viewer;

void scr_widget_destroy(void* widget); //any widget, deallocs ram...

void scr_scrollarea(unsigned char x,unsigned char y, unsigned char xlen,
		         unsigned char ylen);

void scr_clrarea(unsigned char x,unsigned char y, unsigned char xlen,
		         unsigned char ylen);
//size includes corners
void scr_drawbox(unsigned char x,unsigned char y, unsigned char xlen,
	         unsigned char ylen);
//------------------------------------------------------------------
Scr_label* scr_label_create(unsigned char x, unsigned char y,
	unsigned char len, char* text);

void scr_label_draw(Scr_label* label); //redraw label
//------------------------------------------------------------------
Scr_mlabel* scr_mlabel_create(unsigned char x, unsigned char y,
	unsigned char xlen, unsigned char ylen, char** lines);
void scr_mlabel_draw(Scr_mlabel* mlabel); //redraw mlabel
//------------------------------------------------------------------
Scr_menu* scr_menu_create(unsigned char x, unsigned char y, char** choices,
	unsigned char numchoices);
void scr_menu_draw(Scr_menu* menu);
int scr_menu_inject(Scr_menu* menu,int thechar);
//------------------------------------------------------------------
Scr_edit* scr_edit_create(unsigned char x, unsigned char y, unsigned char len, 
	char* text);
void scr_edit_draw(Scr_edit* edit);
int scr_edit_inject(Scr_edit* edit,int thechar);
//------------------------------------------------------------------
Scr_memo* scr_memo_create(unsigned char x, unsigned char y, unsigned char xlen, 
	unsigned char ylen, char** lines);
void scr_memo_draw(Scr_memo* memo);
int scr_memo_inject(Scr_memo* memo,int thechar);
//------------------------------------------------------------------
Scr_checkgrp* scr_checkgrp_create(unsigned char x, unsigned char y, 
	unsigned char numchoices, 
	char** choices,	char* chosens);
void scr_checkgrp_draw(Scr_checkgrp* checkgrp);
int scr_checkgrp_inject(Scr_checkgrp* checkgrp, int thechar);
//------------------------------------------------------------------
Scr_hprogrbar* scr_hprogrbar_create(unsigned char x, unsigned char y, 
	unsigned char len);
void scr_hprogrbar_draw(Scr_hprogrbar* bar);
//------------------------------------------------------------------
Scr_vprogrbar* scr_vprogrbar_create(unsigned char x, unsigned char y, 
	unsigned char len);
void scr_vprogrbar_draw(Scr_vprogrbar* bar);
//------------------------------------------------------------------
Scr_vscrollbar* scr_vscrollbar_create(unsigned char x, unsigned char y, 
	unsigned char len);
void scr_vscrollbar_draw(Scr_vscrollbar* bar);
Scr_hscrollbar* scr_hscrollbar_create(unsigned char x, unsigned char y, 
	unsigned char len);
void scr_hscrollbar_draw(Scr_hscrollbar* bar);
//------------------------------------------------------------------
Scr_viewer* scr_viewer_create(unsigned char x, unsigned char y,
	unsigned char xlen, unsigned char ylen, char** lines, 
	unsigned int numlines);
void scr_viewer_draw(Scr_viewer* viewer);
int scr_viewer_inject(Scr_viewer* viewer,int thechar);

#endif
