//WINAPI driver

static HANDLE Scr_myhin;
static HANDLE Scr_myhout;

void scr_refresh(void) {
COORD c;
scr_refresh_ind_linewise();
c.X = scr_curs_x;
c.Y = scr_curs_y;
SetConsoleCursorPosition(Scr_myhout, c);
}

//inits just the character device, not any memory
void scr_initscr_low(void)
{
    /* Create handles for console input and output.  This should
     * work even if stdin and stdout are redirected, or we're detached.
     */
    Scr_myhin = CreateFile("CONIN$", GENERIC_READ | GENERIC_WRITE,
                    FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                       OPEN_EXISTING, 0, NULL);
    Scr_myhout = CreateFile("CONOUT$", GENERIC_READ | GENERIC_WRITE,
                       FILE_SHARE_WRITE | FILE_SHARE_READ, NULL,
                       OPEN_EXISTING, 0, NULL);

}

void scr_stopscr_low(void) //deinits just the character device
{
CloseHandle(Scr_myhin);
CloseHandle(Scr_myhout);
}

void scr_cursor(unsigned char x,unsigned char y)
{
scr_curs_y=y;
scr_curs_x=x;
}

void scr_hidecursor(void) 
{
CONSOLE_CURSOR_INFO info;
info.dwSize = 1;
info.bVisible = FALSE;
SetConsoleCursorInfo(GetStdHandle(STD_OUTPUT_HANDLE), &info);
}

void scr_showcursor(void)
{
CONSOLE_CURSOR_INFO info;
info.dwSize = 1;
info.bVisible = TRUE;
SetConsoleCursorInfo(GetStdHandle(STD_OUTPUT_HANDLE), &info);
}

int scr_getch_nowait(void) 
{
int tmpchar;
INPUT_RECORD inp;
DWORD nread, numevents, f;
DWORD kbdmode;

/* Get the number of pending input records */
GetNumberOfConsoleInputEvents(Scr_myhin, &numevents);

if (numevents == 0) return SCR_KEY_NONE;
if (GetConsoleMode(Scr_myhin , &kbdmode) != TRUE) return -1;
if (SetConsoleMode(Scr_myhin, 0 ) != TRUE) return -1;

tmpchar=SCR_KEY_NONE;
/* Search for a KEY event */
for (f=0;f<numevents;f++) 
    {
    /* READ the pending records one-by-one*/
    if (ReadConsoleInput(Scr_myhin, &inp, 1, &nread) != TRUE) return -1;
        if ((inp.EventType==KEY_EVENT) && inp.Event.KeyEvent.bKeyDown)
        {
        tmpchar = inp.Event.KeyEvent.wVirtualKeyCode + 0x100; //he-he
        if (inp.Event.KeyEvent.uChar.AsciiChar>0)
              {
              tmpchar = inp.Event.KeyEvent.uChar.AsciiChar;
		switch (tmpchar)
		     {
		     case 27:
	             tmpchar=SCR_KEY_ESC;
		     break;
		     case 13:
	             tmpchar=SCR_KEY_ENTER;
		     break;
		     case 8:
	             tmpchar=SCR_KEY_BACKSPACE;
		     break;
		     case 9:
	             tmpchar=SCR_KEY_TAB;
		     break;
		     default: //untranslated
		     break;
		     };
              };
        break;           
        };
    }

/* Change the keyboard back to its original mode. */
if (SetConsoleMode(Scr_myhin, kbdmode) != TRUE) return -1;

return tmpchar;
}


int scr_getch(void) 
{
int tmpchar;
INPUT_RECORD inp;
DWORD nread;
DWORD kbdmode;

if (GetConsoleMode(Scr_myhin , &kbdmode) != TRUE)
{
return -1;
};

if (SetConsoleMode(Scr_myhin, 0 ) != TRUE)
{
return -1;
};

    /* Get keyboard events until a recognizable one appears.
     */
    for (;;)
    {
        if (ReadConsoleInput(Scr_myhin, &inp, 1, &nread) != TRUE)
        {
            tmpchar = -1;
            break;
        }

        if ((inp.EventType==KEY_EVENT) && inp.Event.KeyEvent.bKeyDown)
        {
            tmpchar = inp.Event.KeyEvent.wVirtualKeyCode + 0x100; //he-he
            if (inp.Event.KeyEvent.uChar.AsciiChar>0)
              {
              tmpchar = inp.Event.KeyEvent.uChar.AsciiChar;
		switch (tmpchar)
		     {
		     case 27:
	             tmpchar=SCR_KEY_ESC;
		     break;
		     case 13:
	             tmpchar=SCR_KEY_ENTER;
		     break;
		     case 8:
	             tmpchar=SCR_KEY_BACKSPACE;
		     break;
		     case 9:
	             tmpchar=SCR_KEY_TAB;
		     break;
		     default: //untranslated
		     break;
		     };
              };
            break;           
        };
    };

    /* Change the keyboard back to its original mode. */
if (SetConsoleMode(Scr_myhin, kbdmode) != TRUE)
{
return -1;
}

return tmpchar;
}

unsigned char scr_maxx(void) 
{
return 80;
} //screen width

unsigned char scr_maxy(void) //screen height
{
return 25;
}

//type conversion functions:
unsigned char scr_char2char(scr_char ch)
{
return ch.Char.AsciiChar;
}

scr_attr scr_char2attr(scr_char ch)
{
return ch.Attributes;
}

scr_char scr_to_scr_char(unsigned char ch, scr_attr attr)
{
CHAR_INFO tmpchar;
tmpchar.Char.AsciiChar=ch;
tmpchar.Attributes=attr;
return tmpchar;
}

scr_attr scr_color2attr(unsigned int color)
{
return color;
}

unsigned char scr_charcomp(scr_char ch1,scr_char ch2)
{
if (ch1.Char.AsciiChar != ch2.Char.AsciiChar) return 0;
if (ch1.Attributes != ch2.Attributes) return 0;
return 1;
}

//for linewise method:
void scr_drv_showline(unsigned char x, unsigned char y, 
	          unsigned char xlen, scr_char* line)
{
unsigned char f;
COORD bs;               /* buffer size */
COORD bc;               /* buffer coordinate */
SMALL_RECT wr;          /* write region */

bs.X=xlen;
bs.Y=1;
bc.X = 0;
bc.Y = 0;
wr.Top=y;
wr.Bottom=y;
wr.Left=x;
wr.Right=x+xlen-1;
(void) WriteConsoleOutput(Scr_myhout, line, bs, bc, &wr);
}

//for 'teletype' method:
void scr_drv_home(void)  //position to 0:0
{
}

void scr_drv_skiplines(unsigned char lines) //skip so much lines
{
}

void scr_drv_skipchars(unsigned char chars) //skip so much chars
{
}

void scr_drv_showstring(unsigned char len, scr_char* line)
{
}

